@@ -14,9 +14,6 @@ class ModelObject:
     def __hash__(self):
         return self.id
 
-    def __gt__(self, other):
-        return self.id > other.id
-
     def __eq__(self, other):
         try:
             return self.id == other.id
