@@ -7,19 +7,18 @@ from typing import Tuple, Optional
 
 import numpy as np
 
+from autoconf import conf
 from autofit import exc
 from autofit.mapper import model
-from autofit.mapper import model_mapper
 from autofit.mapper.model import AbstractModel
 from autofit.mapper.prior.deferred import DeferredArgument
 from autofit.mapper.prior.prior import GaussianPrior
 from autofit.mapper.prior.prior import TuplePrior, Prior, WidthModifier, Limits
-from autofit.mapper.prior_model import collection
-from autofit.mapper.prior_model import dimension_type as dim
 from autofit.mapper.prior_model.attribute_pair import DeferredNameValue
 from autofit.mapper.prior_model.attribute_pair import cast_collection, PriorNameValue, InstanceNameValue
 from autofit.mapper.prior_model.recursion import DynamicRecursionCache
 from autofit.mapper.prior_model.util import PriorModelNameValue
+from autofit.text import formatter as frm
 from autofit.text.formatter import TextFormatter
 
 
@@ -97,6 +96,7 @@ class AbstractPriorModel(AbstractModel):
             obj = object.__new__(PriorModel)
             obj.__init__(t, **kwargs)
         elif isinstance(t, list) or isinstance(t, dict):
+            from autofit.mapper.prior_model import collection
             obj = object.__new__(
                 collection.CollectionPriorModel
             )
@@ -105,14 +105,71 @@ class AbstractPriorModel(AbstractModel):
             obj = t
         return obj
 
+    def take_attributes(
+            self,
+            source: object
+    ):
+        """
+        Take all attributes with a matching path from the source prior model.
+
+        For example, if this prior model has a prior "one" and a matching prior
+        is found associated with the source model then that attribute is attached
+        to this model.
+
+        If no matching attribute is found nothing happens.
+
+        Parameters
+        ----------
+        source
+            An instance or prior model from a previous phase from which attributes
+            are passed to this model.
+        """
+
+        def assert_no_assertions(obj):
+            if len(getattr(obj, "_assertions", [])) > 0:
+                raise AssertionError(
+                    "take_attributes cannot be called once assertions have been added to the target"
+                )
+
+        assert_no_assertions(self)
+
+        for path, _ in sum(map(
+                self.path_instance_tuples_for_class,
+                (Prior, float, TuplePrior)),
+                []
+        ):
+            try:
+                item = copy.copy(source)
+                if isinstance(item, dict):
+                    from autofit.mapper.prior_model.collection import CollectionPriorModel
+                    item = CollectionPriorModel(item)
+                for attribute in path:
+                    item = copy.copy(
+                        getattr(
+                            item,
+                            attribute
+                        )
+                    )
+
+                target = self
+                for attribute in path[:-1]:
+                    target = getattr(target, attribute)
+                    assert_no_assertions(target)
+
+                setattr(target, path[-1], item)
+            except AttributeError:
+                pass
+
     def instance_from_unit_vector(self, unit_vector, assert_priors_in_limits=True):
         """
-        Creates a ModelInstance, which has an attribute and class instance corresponding
-        to every PriorModel attributed to this instance.
+        Returns a ModelInstance, which has an attribute and class instance corresponding
+        to every `PriorModel` attributed to this instance.
         This method takes as input a unit vector of parameter values, converting each to
         physical values via their priors.
         Parameters
         ----------
+        assert_priors_in_limits
+            If true then an exception is thrown if priors fall outside defined limits
         unit_vector: [float]
             A unit hypercube vector that is mapped to an instance of physical values via the priors.
         Returns
@@ -253,8 +310,8 @@ class AbstractPriorModel(AbstractModel):
             assert_priors_in_limits=True
     ):
         """
-        Creates a ModelInstance, which has an attribute and class instance corresponding
-        to every PriorModel attributed to this instance.
+        Returns a ModelInstance, which has an attribute and class instance corresponding
+        to every `PriorModel` attributed to this instance.
         This method takes as input a physical vector of parameter values, thus omitting
         the use of priors.
         Parameters
@@ -262,7 +319,7 @@ class AbstractPriorModel(AbstractModel):
         vector: [float]
             A vector of physical parameter values that is mapped to an instance.
         assert_priors_in_limits
-            If True it is checked that the physical values of priors are within set limits
+            If `True` it is checked that the physical values of priors are within set limits
         Returns
         -------
         model_instance : autofit.mapper.model.ModelInstance
@@ -283,7 +340,7 @@ class AbstractPriorModel(AbstractModel):
 
     def mapper_from_partial_prior_arguments(self, arguments):
         """
-        Creates a new model mapper from a dictionary mapping_matrix existing priors to
+        Returns a new model mapper from a dictionary mapping_matrix existing priors to
         new priors, keeping existing priors where no mapping is provided.
         Parameters
         ----------
@@ -299,7 +356,7 @@ class AbstractPriorModel(AbstractModel):
 
     def mapper_from_prior_arguments(self, arguments):
         """
-        Creates a new model mapper from a dictionary mapping_matrix existing priors to
+        Returns a new model mapper from a dictionary mapping_matrix existing priors to
         new priors.
         Parameters
         ----------
@@ -328,11 +385,12 @@ class AbstractPriorModel(AbstractModel):
             tuples,
             a=None,
             r=None,
+            use_errors=True,
+            use_widths=True,
             no_limits=False
     ):
         """
-        Creates a new model mapper from a list of floats describing the mean values
-        of gaussian priors. The widths of the new priors are taken from the
+        The widths of the new priors are taken from the
         width_config. The new gaussian priors must be provided in the same order as
         the priors associated with model.
         If a is not None then all priors are created with an absolute width of a.
@@ -340,11 +398,20 @@ class AbstractPriorModel(AbstractModel):
         Parameters
         ----------
         no_limits
-            If True generated priors have infinite limits
+            If `True` generated priors have infinite limits
         r
             The relative width to be assigned to gaussian priors
         a
+            print(tuples[i][1], width)
             The absolute width to be assigned to gaussian priors
+        use_errors : bool
+            If True, the passed errors of the model components estimated in a previous `NonLinearSearch` (computed
+            at the prior_passer.sigma value) are used to set the pass Gaussian Prior sigma value (if both width and
+            passed errors are used, the maximum of these two values are used).
+        use_widths : bool
+            If True, the minimum prior widths specified in the prior configs of the model components are used to
+            set the passed Gaussian Prior sigma value (if both widths and passed errors are used, the maximum of
+            these two values are used).
         tuples
             A list of tuples each containing the mean and width of a prior
         Returns
@@ -392,9 +459,19 @@ class AbstractPriorModel(AbstractModel):
                 except exc.PriorException:
                     limits = prior.limits
 
+            if use_errors and not use_widths:
+                sigma = tuples[i][1]
+            elif not use_errors and use_widths:
+                sigma = width
+            elif use_errors and use_widths:
+                sigma = max(tuples[i][1], width)
+            else:
+                raise exc.PriorException("use_passed_errors and use_widths are both False, meeaning there is no "
+                                         "way to pass priors to set up the new model's Gaussian Priors.")
+
             arguments[prior] = GaussianPrior(
                 mean,
-                max(tuples[i][1], width),
+                sigma,
                 *limits
             )
 
@@ -402,7 +479,7 @@ class AbstractPriorModel(AbstractModel):
 
     def instance_from_prior_medians(self):
         """
-        Creates a list of physical values from the median values of the priors.
+        Returns a list of physical values from the median values of the priors.
         Returns
         -------
         physical_values : [float]
@@ -438,7 +515,7 @@ class AbstractPriorModel(AbstractModel):
 
     def random_instance(self):
         """
-        Creates a random instance of the model.
+        Returns a random instance of the model.
         """
         return self.instance_from_unit_vector(
             unit_vector=[random() for _ in self.prior_tuples]
@@ -459,7 +536,7 @@ class AbstractPriorModel(AbstractModel):
         abstract_prior_model
             A concrete child of an abstract prior model
         """
-
+        from autofit.mapper.prior_model import collection
         if isinstance(instance, list):
             result = collection.CollectionPriorModel(
                 [
@@ -468,6 +545,7 @@ class AbstractPriorModel(AbstractModel):
                 ]
             )
         elif isinstance(instance, model.ModelInstance):
+            from autofit.mapper import model_mapper
             result = model_mapper.ModelMapper()
             for key, value in instance.dict.items():
                 setattr(
@@ -486,7 +564,7 @@ class AbstractPriorModel(AbstractModel):
                     for key, value in instance.items()
                 }
             )
-        elif isinstance(instance, (dim.DimensionType, np.ndarray)):
+        elif isinstance(instance, np.ndarray):
             return instance
         else:
             from .prior_model import PriorModel
@@ -599,12 +677,14 @@ class AbstractPriorModel(AbstractModel):
             assert_priors_in_limits=True
     ):
         """
-        Create an instance of the model for a set of arguments
+        Returns an instance of the model for a set of arguments
         Parameters
         ----------
         assert_priors_in_limits
-            If true it is asserted that the physical values that replace piors are
-            within their limits
+            If true it is asserted that the physical values that replace priors are
+            within their limits.
+            If ignore_prior_limits is true in configuration then prior limits are
+            ignored regardless.
         arguments: {Prior: float}
             Dictionary mapping_matrix priors to attribute analysis_path and value pairs
         Returns
@@ -615,7 +695,7 @@ class AbstractPriorModel(AbstractModel):
             raise exc.PriorException(
                 "All promises must be populated prior to instantiation"
             )
-        if assert_priors_in_limits:
+        if assert_priors_in_limits and not conf.instance["general"]["model"]["ignore_prior_limits"]:
             for prior, value in arguments.items():
                 if isinstance(value, Number):
                     prior.assert_within_limits(value)
@@ -643,6 +723,19 @@ class AbstractPriorModel(AbstractModel):
     def priors(self):
         return [prior_tuple.prior for prior_tuple in self.prior_tuples]
 
+    @property
+    def _prior_id_map(self):
+        return {
+            prior.id: prior
+            for prior
+            in self.priors
+        }
+
+    def prior_with_id(self, prior_id):
+        return self._prior_id_map[
+            prior_id
+        ]
+
     def name_for_prior(self, prior):
         for prior_model_name, prior_model in self.direct_prior_model_tuples:
             prior_name = prior_model.name_for_prior(prior)
@@ -741,7 +834,7 @@ class AbstractPriorModel(AbstractModel):
         parameter of the overall model.
         This information is extracted from each priors *model_info* property.
         """
-        from autofit.mapper.prior import AbstractPromise
+        from autofit.mapper.prior.promise import AbstractPromise
         formatter = TextFormatter()
 
         for t in self.path_instance_tuples_for_class((
@@ -752,9 +845,9 @@ class AbstractPriorModel(AbstractModel):
         return formatter.text
 
     @property
-    def parameter_names(self) -> [str]:
+    def model_component_and_parameter_names(self) -> [str]:
         """The param_names vector is a list each parameter's analysis_path, and is used
-        for *GetDist* visualization.
+        for *corner.py* visualization.
         The parameter names are determined from the class instance names of the
         model_mapper. Latex tags are properties of each model class."""
         return [
@@ -765,6 +858,61 @@ class AbstractPriorModel(AbstractModel):
             in self.prior_tuples_ordered_by_id
         ]
 
+    @property
+    def parameter_names(self) -> [str]:
+        """The param_names vector is a list each parameter's analysis_path, and is used
+        for *corner.py* visualization.
+        The parameter names are determined from the class instance names of the
+        model_mapper. Latex tags are properties of each model class."""
+        return [parameter_name[-1] for parameter_name in self.unique_prior_paths]
+
+    @property
+    def parameter_labels(self) -> [str]:
+        """
+        Returns a list of the label of every parameter in a model.
+
+        This is used for displaying model results as text and for visualization with *corner.py*.
+
+        The parameter labels are defined for every parameter of every model component in the config files label.ini and
+        label_format.ini.
+        """
+
+        parameter_labels = []
+
+        for parameter_name in self.parameter_names:
+            parameter_label = frm.convert_name_to_label(parameter_name=parameter_name, name_to_label=True)
+            parameter_labels.append(parameter_label)
+
+        return parameter_labels
+
+    @property
+    def subscripts(self) -> [str]:
+        """
+        Returns a list of the model component subscripts of every parameter in a model.
+
+        This is used for displaying model results as text and for visualization with *corner.py*.
+
+        The class subscript labels are defined for every model component in the config file notation/label.ini.
+        """
+
+        subscripts = []
+
+        for prior_name, prior in self.prior_tuples_ordered_by_id:
+            cls = self.prior_class_dict[prior]
+            try:
+                subscript = conf.instance[
+                    "notation"
+                ][
+                    "label"
+                ][
+                    "subscript"
+                ].family(cls)
+            except KeyError:
+                subscript = prior_name[0]
+            subscripts.append(subscript)
+
+        return subscripts
+
 
 def transfer_classes(instance, mapper, model_classes=None):
     """
