@@ -2,10 +2,8 @@ import copy
 import inspect
 import logging
 
-from typing_inspect import is_tuple_type
-
 from autofit.mapper.model_object import ModelObject
-from autofit.mapper.prior import TuplePrior, Prior
+from autofit.mapper.prior.prior import TuplePrior, Prior
 from autofit.mapper.prior.deferred import DeferredInstance
 from autofit.mapper.prior.promise import Promise
 from autofit.mapper.prior_model.abstract import AbstractPriorModel
@@ -23,6 +21,13 @@ class PriorModel(AbstractPriorModel):
     def name(self):
         return self.cls.__name__
 
+    def __str__(self):
+        prior_string = ", ".join(map(str, self.prior_tuples))
+        return f"{self.name} {prior_string}"
+
+    def __repr__(self):
+        return f"<{self.__class__.__name__} {self}>"
+
     def as_model(self):
         return PriorModel(self.cls)
 
@@ -112,14 +117,6 @@ class PriorModel(AbstractPriorModel):
                     )
 
                     setattr(self, arg, AnnotationPriorModel(spec, cls, arg))
-                elif is_tuple_type(spec):
-                    tuple_prior = TuplePrior()
-                    for i, tuple_arg in enumerate(spec.__args__):
-                        attribute_name = "{}_{}".format(arg, i)
-                        setattr(
-                            tuple_prior, attribute_name, self.make_prior(attribute_name)
-                        )
-                    setattr(self, arg, tuple_prior)
                 else:
                     setattr(self, arg, PriorModel(annotations[arg]))
             else:
@@ -139,7 +136,7 @@ class PriorModel(AbstractPriorModel):
 
     def make_prior(self, attribute_name):
         """
-        Create a prior for an attribute of a class with a given name. The prior is
+        Returns a prior for an attribute of a class with a given name. The prior is
         created by searching the default prior config for the attribute.
 
         Entries in configuration with a u become uniform priors; with a g become
@@ -216,7 +213,7 @@ class PriorModel(AbstractPriorModel):
     @check_assertions
     def _instance_for_arguments(self, arguments: {ModelObject: object}):
         """
-        Create an instance of the associated class for a set of arguments
+        Returns an instance of the associated class for a set of arguments
 
         Parameters
         ----------
@@ -244,10 +241,20 @@ class PriorModel(AbstractPriorModel):
                 prior_model_tuple.name
             ] = prior_model.instance_for_arguments(arguments)
 
+        prior_arguments = dict()
+
+        for name, prior in self.direct_prior_tuples:
+            try:
+                prior_arguments[name] = arguments[prior]
+            except KeyError as e:
+                raise KeyError(
+                    f"No argument given for prior {name}"
+                ) from e
+
         constructor_arguments = {
             **attribute_arguments,
             **model_arguments,
-            **{t.name: arguments[t.prior] for t in self.direct_prior_tuples},
+            **prior_arguments,
         }
 
         if self.is_deferred_arguments:
@@ -279,7 +286,7 @@ class PriorModel(AbstractPriorModel):
 
     def gaussian_prior_model_for_arguments(self, arguments):
         """
-        Create a new instance of model mapper with a set of Gaussian priors based on \
+        Returns a new instance of model mapper with a set of Gaussian priors based on \
         tuples provided by a previous nonlinear search.
 
         Parameters
