@@ -1,45 +1,43 @@
-import logging
-
-import numpy as np
-
-from autofit import exc
+from autoconf import conf
 from autofit.mapper.prior_model.abstract import AbstractPriorModel
-from autofit.non_linear.samples import NestSamples
-from autofit.non_linear.nest import abstract_nest
 from autofit.non_linear import abstract_search
-
-logger = logging.getLogger(__name__)
+from autofit.non_linear.log import logger
+from autofit.non_linear.nest import abstract_nest
+from autofit.non_linear.paths import convert_paths
+from autofit.non_linear.samples import NestSamples, Sample
 
 
 class MultiNest(abstract_nest.AbstractNest):
+
+    @convert_paths
     def __init__(
-        self,
-        paths=None,
-        sigma=3,
-        n_live_points=None,
-        sampling_efficiency=None,
-        const_efficiency_mode=None,
-        multimodal=None,
-        importance_nested_sampling=None,
-        evidence_tolerance=None,
-        max_modes=None,
-        mode_tolerance=None,
-        max_iter=None,
-        n_iter_before_update=None,
-        null_log_evidence=None,
-        seed=None,
-        verbose=None,
-        resume=None,
-        context=None,
-        write_output=None,
-        log_zero=None,
-        init_MPI=None,
-        terminate_at_acceptance_ratio=None,
-        acceptance_ratio_threshold=None,
-        stagger_resampling_likelihood=None,
+            self,
+            paths=None,
+            prior_passer=None,
+            n_live_points=None,
+            sampling_efficiency=None,
+            const_efficiency_mode=None,
+            multimodal=None,
+            importance_nested_sampling=None,
+            evidence_tolerance=None,
+            max_modes=None,
+            mode_tolerance=None,
+            max_iter=None,
+            n_iter_before_update=None,
+            null_log_evidence=None,
+            seed=None,
+            verbose=None,
+            resume=None,
+            context=None,
+            write_output=None,
+            log_zero=None,
+            init_MPI=None,
+            terminate_at_acceptance_ratio=None,
+            acceptance_ratio_threshold=None,
+            stagger_resampling_likelihood=None,
     ):
         """
-        Class to setup and run a MultiNest non-linear search.
+        A MultiNest non-linear search.
 
         For a full description of MultiNest and its Python wrapper PyMultiNest, checkout its Github and documentation
         webpages:
@@ -51,11 +49,9 @@ class MultiNest(abstract_nest.AbstractNest):
         Parameters
         ----------
         paths : af.Paths
-            A class that manages all paths, e.g. where the phase outputs are stored, the non-linear search samples,
-            backups, etc.
-        sigma : float
-            The error-bound value that linked Gaussian prior withs are computed using. For example, if sigma=3.0,
-            parameters will use Gaussian Priors with widths coresponding to errors estimated at 3 sigma confidence.
+            Manages all paths, e.g. where the search outputs are stored, the samples, etc.
+        prior_passer : af.PriorPasser
+            Controls how priors are passed from the results of this `NonLinearSearch` to a subsequent non-linear search.
         n_live_points : int
             The number of live points used to sample non-linear parameter space. More points provides a more thorough
             sampling of parameter space, at the expense of taking longer to run. The number of live points required for
@@ -100,7 +96,7 @@ class MultiNest(abstract_nest.AbstractNest):
         verbose : bool
             Whether MultiNest prints messages.
         resume : bool
-            If True and existing results are found at the output path, MultiNest will resume that run. If False,
+            If `True` and existing results are found at the output path, MultiNest will resume that run. If False,
             MultiNest will start a new run.
         context : None
             Not used by PyAutoFit.
@@ -111,89 +107,95 @@ class MultiNest(abstract_nest.AbstractNest):
         init_MPI : None
             MPI not supported by PyAutoFit for MultiNest.
         terminate_at_acceptance_ratio : bool
-            If *True*, the sampler will automatically terminate when the acceptance ratio falls behind an input
+            If `True`, the sampler will automatically terminate when the acceptance ratio falls behind an input
             threshold value (see *Nest* for a full description of this feature).
         acceptance_ratio_threshold : float
             The acceptance ratio threshold below which sampling terminates if *terminate_at_acceptance_ratio* is
-            *True* (see *Nest* for a full description of this feature).
+            `True` (see *Nest* for a full description of this feature).
         """
 
         self.n_live_points = (
-            self.config("search", "n_live_points", int)
+            self._config("search", "n_live_points")
             if n_live_points is None
             else n_live_points
         )
         self.sampling_efficiency = (
-            self.config("search", "sampling_efficiency", float)
+            self._config("search", "sampling_efficiency")
             if sampling_efficiency is None
             else sampling_efficiency
         )
         self.const_efficiency_mode = (
-            self.config("search", "const_efficiency_mode", bool)
+            self._config("search", "const_efficiency_mode")
             if const_efficiency_mode is None
             else const_efficiency_mode
         )
         self.evidence_tolerance = (
-            self.config("search", "evidence_tolerance", float)
+            self._config("search", "evidence_tolerance")
             if evidence_tolerance is None
             else evidence_tolerance
         )
+
+        if self.evidence_tolerance <= 0.0:
+            self.evidence_tolerance = 0.8
+
         self.multimodal = (
-            multimodal or self.config("search", "multimodal", bool)
+            multimodal or self._config("search", "multimodal")
             if multimodal is None
             else multimodal
         )
         self.importance_nested_sampling = (
-            self.config("search", "importance_nested_sampling", bool)
+            self._config("search", "importance_nested_sampling")
             if importance_nested_sampling is None
             else importance_nested_sampling
         )
         self.max_modes = (
-            self.config("search", "max_modes", int) if max_modes is None else max_modes
+            self._config("search", "max_modes") if max_modes is None else max_modes
         )
         self.mode_tolerance = (
-            self.config("search", "mode_tolerance", float)
+            self._config("search", "mode_tolerance")
             if mode_tolerance is None
             else mode_tolerance
         )
         self.max_iter = (
-            self.config("search", "max_iter", int) if max_iter is None else max_iter
+            self._config("search", "max_iter") if max_iter is None else max_iter
         )
         self.n_iter_before_update = (
-            self.config("settings", "n_iter_before_update", int)
+            self._config("settings", "n_iter_before_update")
             if n_iter_before_update is None
             else n_iter_before_update
         )
         self.null_log_evidence = (
-            self.config("settings", "null_log_evidence", float)
+            self._config("settings", "null_log_evidence")
             if null_log_evidence is None
             else null_log_evidence
         )
-        self.seed = self.config("settings", "seed", int) if seed is None else seed
+        self.seed = self._config("settings", "seed") if seed is None else seed
         self.verbose = (
-            self.config("settings", "verbose", bool) if verbose is None else verbose
+            self._config("settings", "verbose") if verbose is None else verbose
         )
         self.resume = (
-            self.config("settings", "resume", bool) if resume is None else resume
+            self._config("settings", "resume") if resume is None else resume
         )
         self.context = (
-            self.config("settings", "context", int) if context is None else context
+            self._config("settings", "context") if context is None else context
         )
         self.write_output = (
-            self.config("settings", "write_output", bool)
+            self._config("settings", "write_output")
             if write_output is None
             else write_output
         )
         self.log_zero = (
-            self.config("settings", "log_zero", float) if log_zero is None else log_zero
+            self._config("settings", "log_zero")
+            if log_zero is None
+            else log_zero
         )
         self.init_MPI = (
-            self.config("settings", "init_MPI", bool) if init_MPI is None else init_MPI
+            self._config("settings", "init_MPI") if init_MPI is None else init_MPI
         )
 
         super().__init__(
             paths=paths,
-            sigma=sigma,
+            prior_passer=prior_passer,
             terminate_at_acceptance_ratio=terminate_at_acceptance_ratio,
             acceptance_ratio_threshold=acceptance_ratio_threshold,
             stagger_resampling_likelihood=stagger_resampling_likelihood,
@@ -201,10 +203,44 @@ class MultiNest(abstract_nest.AbstractNest):
 
         logger.debug("Creating MultiNest NLO")
 
-    def _fit(self, model: AbstractPriorModel, analysis) -> abstract_search.Result:
+    class Fitness(abstract_nest.AbstractNest.Fitness):
+
+        def __init__(self, paths, model, analysis, samples_from_model, stagger_resampling_likelihood,
+                     terminate_at_acceptance_ratio,
+                     acceptance_ratio_threshold, log_likelihood_cap=None, pool_ids=None):
+
+            super().__init__(paths=paths, model=model, analysis=analysis,
+                             samples_from_model=samples_from_model,
+                             stagger_resampling_likelihood=stagger_resampling_likelihood,
+                             terminate_at_acceptance_ratio=terminate_at_acceptance_ratio,
+                             acceptance_ratio_threshold=acceptance_ratio_threshold,
+                             log_likelihood_cap=log_likelihood_cap,
+                             pool_ids=pool_ids)
+
+            should_update_sym = conf.instance["non_linear"]["nest"]["MultiNest"]["updates"]["should_update_sym"]
+
+            self.should_update_sym = abstract_search.IntervalCounter(should_update_sym)
+
+        def fit_instance(self, instance):
+
+            if self.should_update_sym():
+                self.paths.copy_from_sym()
+
+            log_likelihood = self.analysis.log_likelihood_function(instance=instance)
+
+            if self.log_likelihood_cap is not None:
+                if log_likelihood > self.log_likelihood_cap:
+                    log_likelihood = self.log_likelihood_cap
+
+            if log_likelihood > self.max_log_likelihood:
+                self.max_log_likelihood = log_likelihood
+
+            return log_likelihood
+
+    def _fit(self, model: AbstractPriorModel, analysis, log_likelihood_cap=None) -> abstract_search.Result:
         """
         Fit a model using MultiNest and the Analysis class which contains the data and returns the log likelihood from
-        instances of the model, which the non-linear search seeks to maximize.
+        instances of the model, which the `NonLinearSearch` seeks to maximize.
 
         Parameters
         ----------
@@ -212,7 +248,7 @@ class MultiNest(abstract_nest.AbstractNest):
             The model which generates instances for different points in parameter space.
         analysis : Analysis
             Contains the data and the log likelihood function which fits an instance of the model to the data, returning
-            the log likelihood the non-linear search maximizes.
+            the log likelihood the `NonLinearSearch` maximizes.
 
         Returns
         -------
@@ -236,6 +272,8 @@ class MultiNest(abstract_nest.AbstractNest):
 
         import pymultinest
 
+        logger.info("Beginning MultiNest non-linear search. ")
+
         pymultinest.run(
             fitness_function,
             prior,
@@ -260,46 +298,47 @@ class MultiNest(abstract_nest.AbstractNest):
             max_iter=self.max_iter,
             init_MPI=self.init_MPI,
         )
+        self.paths.copy_from_sym()
 
     @property
     def tag(self):
         """Tag the output folder of the PySwarms non-linear search, according to the number of particles and
         parameters defining the search strategy."""
 
-        name_tag = self.config("tag", "name")
+        name_tag = self._config("tag", "name")
         n_live_points_tag = (
-            f"{self.config('tag', 'n_live_points')}_{self.n_live_points}"
+            f"{self._config('tag', 'n_live_points')}_{self.n_live_points}"
         )
         sampling_efficiency_tag = (
-            f"{self.config('tag', 'sampling_efficiency')}_{self.sampling_efficiency}"
+            f"{self._config('tag', 'sampling_efficiency')}_{self.sampling_efficiency}"
         )
         if self.const_efficiency_mode:
             const_efficiency_mode_tag = (
-                f"_{self.config('tag', 'const_efficiency_mode')}"
+                f"_{self._config('tag', 'const_efficiency_mode')}"
             )
         else:
             const_efficiency_mode_tag = ""
         if self.multimodal:
-            multimodal_tag = f"_{self.config('tag', 'multimodal')}"
+            multimodal_tag = f"_{self._config('tag', 'multimodal')}"
         else:
             multimodal_tag = ""
         if self.importance_nested_sampling:
             importance_nested_sampling_tag = (
-                f"_{self.config('tag', 'importance_nested_sampling')}"
+                f"_{self._config('tag', 'importance_nested_sampling')}"
             )
         else:
             importance_nested_sampling_tag = ""
 
-        return f"{name_tag}__{n_live_points_tag}_{sampling_efficiency_tag}{const_efficiency_mode_tag}{multimodal_tag}{importance_nested_sampling_tag}"
+        return f"{name_tag}[{n_live_points_tag}_{sampling_efficiency_tag}{const_efficiency_mode_tag}{multimodal_tag}{importance_nested_sampling_tag}]"
 
-    def copy_with_name_extension(self, extension, remove_phase_tag=False):
-        """Copy this instance of the multinest non-linear search with all associated attributes.
+    def copy_with_name_extension(self, extension, path_prefix=None, remove_phase_tag=False):
+        """Copy this instance of the multinest `NonLinearSearch` with all associated attributes.
 
-        This is used to set up the non-linear search on phase extensions."""
+        This is used to set up the `NonLinearSearch` on phase extensions."""
         copy = super().copy_with_name_extension(
-            extension=extension, remove_phase_tag=remove_phase_tag
+            extension=extension, path_prefix=path_prefix, remove_phase_tag=remove_phase_tag
         )
-        copy.sigma = self.sigma
+        copy.prior_passer = self.prior_passer
         copy.importance_nested_sampling = self.importance_nested_sampling
         copy.multimodal = self.multimodal
         copy.const_efficiency_mode = self.const_efficiency_mode
@@ -323,8 +362,8 @@ class MultiNest(abstract_nest.AbstractNest):
         copy.stagger_resampling_likelihood = self.stagger_resampling_likelihood
         return copy
 
-    def samples_from_model(self, model: AbstractPriorModel):
-        """Create a *Samples* object from this non-linear search's output files on the hard-disk and model.
+    def samples_via_sampler_from_model(self, model: AbstractPriorModel):
+        """Create a `Samples` object from this non-linear search's output files on the hard-disk and model.
 
         For MulitNest, this requires us to load:
 
@@ -367,18 +406,22 @@ class MultiNest(abstract_nest.AbstractNest):
 
         return NestSamples(
             model=model,
-            parameters=parameters,
-            log_likelihoods=log_likelihoods,
-            log_priors=log_priors,
-            weights=weights,
+            samples=Sample.from_lists(
+                parameters=parameters,
+                log_likelihoods=log_likelihoods,
+                log_priors=log_priors,
+                weights=weights,
+                model=model
+            ),
             total_samples=total_samples,
             log_evidence=log_evidence,
             number_live_points=self.n_live_points,
+            time=self.timer.time,
         )
 
 
 def parameters_from_file_weighted_samples(
-    file_weighted_samples, prior_count
+        file_weighted_samples, prior_count
 ) -> [[float]]:
     """Open the file "multinest.txt" and extract the parameter values of every accepted live point as a list
     of lists."""
