@@ -1,38 +1,37 @@
-import logging
 import os
-import pyswarms
-import numpy as np
 import pickle
 
-from autofit import exc
-from autofit.non_linear.optimize.abstract_optimize import AbstractOptimizer
-from autofit.non_linear.samples import OptimizerSamples
-from autofit.non_linear.abstract_search import Result
+import numpy as np
 
-logger = logging.getLogger(__name__)
+from autofit import exc
+from autofit.mapper.prior_model.abstract import AbstractPriorModel
+from autofit.non_linear.log import logger
+from autofit.non_linear.optimize.abstract_optimize import AbstractOptimizer
+from autofit.non_linear.paths import convert_paths
+from autofit.non_linear.samples import OptimizerSamples, Sample
 
 
 class AbstractPySwarms(AbstractOptimizer):
     def __init__(
-        self,
-        paths=None,
-        sigma=3,
-        n_particles=None,
-        iters=None,
-        cognitive=None,
-        social=None,
-        inertia=None,
-        ftol=None,
-        initializer=None,
-        iterations_per_update=None,
-        number_of_cores=None,
+            self,
+            paths=None,
+            prior_passer=None,
+            n_particles=None,
+            iters=None,
+            cognitive=None,
+            social=None,
+            inertia=None,
+            ftol=None,
+            initializer=None,
+            iterations_per_update=None,
+            number_of_cores=None,
     ):
-        """ Class to setup and run a PySwarms Particle Swarm Optimizer global non-linear search.
+        """
+        A PySwarms Particle Swarm Optimizer global non-linear search.
 
         For a full description of PySwarms, checkout its Github and readthedocs webpages:
 
         https://github.com/ljvmiranda921/pyswarms
-
         https://pyswarms.readthedocs.io/en/latest/index.html
 
         A Global-best Particle Swarm Optimization (gbest PSO) algorithm.
@@ -56,23 +55,26 @@ class AbstractPySwarms(AbstractOptimizer):
 
         Extensions:
 
-        - Allows runs to be terminated and resumed from the point it was terminated. This is achieved by outputting the
-          necessary results (e.g. the points of the particles) during the model-fit after an input number of iterations.
+        Allows runs to be terminated and resumed from the point it was terminated. This is achieved by outputting
+        the necessary results (e.g. the points of the particles) during the model-fit after an input number of
+        iterations.
 
-        - Different options for particle intialization, with the default 'prior' method starting all particles over the
-        priors defined by each parameter.
+        Different options for particle intialization, with the default 'prior' method starting all particles over
+        the priors defined by each parameter.
 
         If you use *PySwarms* as part of a published work, please cite the package following the instructions under the
         *Attribution* section of the GitHub page.
 
+        All remaining attributes are emcee parameters and described at the PySwarms API webpage:
+
+        https://pyswarms.readthedocs.io/en/latest/index.html
+
         Parameters
         ----------
         paths : af.Paths
-            A class that manages all paths, e.g. where the phase outputs are stored, the non-linear search samples,
-            backups, etc.
-        sigma : float
-            The error-bound value that linked Gaussian prior withs are computed using. For example, if sigma=3.0,
-            parameters will use Gaussian Priors with widths coresponding to errors estimated at 3 sigma confidence.
+            Manages all paths, e.g. where the search outputs are stored, the samples, etc.
+        prior_passer : af.PriorPasser
+            Controls how priors are passed from the results of this `NonLinearSearch` to a subsequent non-linear search.
         n_particles : int
             The number of particles in the swarm used to sample parameter space.
         iters : int
@@ -90,42 +92,37 @@ class AbstractPySwarms(AbstractOptimizer):
         number_of_cores : int
             The number of cores Emcee sampling is performed using a Python multiprocessing Pool instance. If 1, a
             pool instance is not created and the job runs in serial.
-
-        All remaining attributes are emcee parameters and described at the PySwarms API webpage:
-
-        https://pyswarms.readthedocs.io/en/latest/index.html
         """
 
-        self.sigma = sigma
-
         self.n_particles = (
-            self.config("search", "n_particles", int)
+            self._config("search", "n_particles")
             if n_particles is None
             else n_particles
         )
-        self.iters = self.config("search", "iters", int) if iters is None else iters
+        self.iters = self._config("search", "iters") if iters is None else iters
 
         self.cognitive = (
-            self.config("search", "cognitive", float)
+            self._config("search", "cognitive")
             if cognitive is None
             else cognitive
         )
         self.social = (
-            self.config("search", "social", float) if social is None else social
+            self._config("search", "social") if social is None else social
         )
         self.inertia = (
-            self.config("search", "inertia", float) if inertia is None else inertia
+            self._config("search", "inertia") if inertia is None else inertia
         )
-        self.ftol = self.config("search", "ftol", float) if ftol is None else ftol
+        self.ftol = self._config("search", "ftol") if ftol is None else ftol
 
         super().__init__(
             paths=paths,
+            prior_passer=prior_passer,
             initializer=initializer,
             iterations_per_update=iterations_per_update,
         )
 
         self.number_of_cores = (
-            self.config("parallel", "number_of_cores", int)
+            self._config("parallel", "number_of_cores")
             if number_of_cores is None
             else number_of_cores
         )
@@ -151,17 +148,17 @@ class AbstractPySwarms(AbstractOptimizer):
             return np.asarray(figures_of_merit)
 
         def figure_of_merit_from_parameters(self, parameters):
-            """The figure of merit is the value that the non-linear search uses to sample parameter space. *PySwarms*
+            """The figure of merit is the value that the `NonLinearSearch` uses to sample parameter space. *PySwarms*
             uses the chi-squared value, which is the -2.0*log_posterior."""
             try:
                 return -2.0 * self.log_posterior_from_parameters(parameters=parameters)
             except exc.FitException:
                 raise exc.FitException
 
-    def _fit(self, model, analysis):
+    def _fit(self, model: AbstractPriorModel, analysis, log_likelihood_cap=None):
         """
         Fit a model using PySwarms and the Analysis class which contains the data and returns the log likelihood from
-        instances of the model, which the non-linear search seeks to maximize.
+        instances of the model, which the `NonLinearSearch` seeks to maximize.
 
         Parameters
         ----------
@@ -169,7 +166,7 @@ class AbstractPySwarms(AbstractOptimizer):
             The model which generates instances for different points in parameter space.
         analysis : Analysis
             Contains the data and the log likelihood function which fits an instance of the model to the data, returning
-            the log likelihood the non-linear search maximizes.
+            the log likelihood the `NonLinearSearch` maximizes.
 
         Returns
         -------
@@ -187,6 +184,8 @@ class AbstractPySwarms(AbstractOptimizer):
             init_pos = self.load_points[-1]
             total_iterations = self.load_total_iterations
 
+            logger.info("Existing PySwarms samples found, resuming non-linear search.")
+
         else:
 
             initial_unit_parameters, initial_parameters, initial_log_posteriors = self.initializer.initial_samples_from_model(
@@ -203,6 +202,8 @@ class AbstractPySwarms(AbstractOptimizer):
 
             total_iterations = 0
 
+            logger.info("No PySwarms samples found, beginning new non-linear search. ")
+
         lower_bounds = []
         upper_bounds = []
 
@@ -237,7 +238,7 @@ class AbstractPySwarms(AbstractOptimizer):
                 total_iterations += iterations
 
                 with open(
-                    f"{self.paths.samples_path}/total_iterations.pickle", "wb"
+                        f"{self.paths.samples_path}/total_iterations.pickle", "wb"
                 ) as f:
                     pickle.dump(total_iterations, f)
 
@@ -245,7 +246,7 @@ class AbstractPySwarms(AbstractOptimizer):
                     pickle.dump(pso.pos_history, f)
 
                 with open(
-                    f"{self.paths.samples_path}/log_posteriors.pickle", "wb"
+                        f"{self.paths.samples_path}/log_posteriors.pickle", "wb"
                 ) as f:
                     pickle.dump([-0.5 * cost for cost in pso.cost_history], f)
 
@@ -262,24 +263,22 @@ class AbstractPySwarms(AbstractOptimizer):
         """Tag the output folder of the PySwarms non-linear search, according to the number of particles and
         parameters defining the search strategy."""
 
-        name_tag = self.config("tag", "name", str)
-        n_particles_tag = f"{self.config('tag', 'n_particles')}_{self.n_particles}"
-        cognitive_tag = f"{self.config('tag', 'cognitive')}_{self.cognitive}"
-        social_tag = f"{self.config('tag', 'social')}_{self.social}"
-        inertia_tag = f"{self.config('tag', 'inertia')}_{self.inertia}"
+        name_tag = self._config("tag", "name")
+        n_particles_tag = f"{self._config('tag', 'n_particles')}_{self.n_particles}"
+        cognitive_tag = f"{self._config('tag', 'cognitive')}_{self.cognitive}"
+        social_tag = f"{self._config('tag', 'social')}_{self.social}"
+        inertia_tag = f"{self._config('tag', 'inertia')}_{self.inertia}"
 
-        return (
-            f"{name_tag}__{n_particles_tag}_{cognitive_tag}_{social_tag}_{inertia_tag}"
-        )
+        return f"{name_tag}[{n_particles_tag}_{cognitive_tag}_{social_tag}_{inertia_tag}]"
 
-    def copy_with_name_extension(self, extension, remove_phase_tag=False):
-        """Copy this instance of the emcee non-linear search with all associated attributes.
+    def copy_with_name_extension(self, extension, path_prefix=None, remove_phase_tag=False):
+        """Copy this instance of the emcee `NonLinearSearch` with all associated attributes.
 
-        This is used to set up the non-linear search on phase extensions."""
+        This is used to set up the `NonLinearSearch` on phase extensions."""
         copy = super().copy_with_name_extension(
-            extension=extension, remove_phase_tag=remove_phase_tag
+            extension=extension, path_prefix=path_prefix, remove_phase_tag=remove_phase_tag
         )
-        copy.sigma = self.sigma
+        copy.prior_passer = self.prior_passer
         copy.n_particles = self.n_particles
         copy.iters = self.iters
         copy.cognitive = self.cognitive
@@ -292,20 +291,21 @@ class AbstractPySwarms(AbstractOptimizer):
 
         return copy
 
-    def fitness_function_from_model_and_analysis(self, model, analysis, pool_ids=None):
+    def fitness_function_from_model_and_analysis(self, model, analysis, log_likelihood_cap=None, pool_ids=None):
 
         return PySwarmsGlobal.Fitness(
             paths=self.paths,
             model=model,
             analysis=analysis,
-            samples_from_model=self.samples_from_model,
+            samples_from_model=self.samples_via_sampler_from_model,
+            log_likelihood_cap=log_likelihood_cap,
             pool_ids=pool_ids,
         )
 
     def sampler_fom_model_and_fitness(self, model, fitness_function):
         raise NotImplementedError()
 
-    def samples_from_model(self, model):
+    def samples_via_sampler_from_model(self, model):
         """Create an *OptimizerSamples* object from this non-linear search's output files on the hard-disk and model.
 
         For PySwarms, all quantities are extracted via pickled states of the particle and cost histories.
@@ -325,18 +325,24 @@ class AbstractPySwarms(AbstractOptimizer):
         ]
         log_posteriors = self.load_log_posteriors
         log_likelihoods = [lp - prior for lp, prior in zip(log_posteriors, log_priors)]
+        weights = len(log_likelihoods) * [1.0]
 
         return OptimizerSamples(
             model=model,
-            parameters=[parameters.tolist()[0] for parameters in self.load_points],
-            log_likelihoods=log_likelihoods,
-            log_priors=log_priors,
+            samples=Sample.from_lists(
+                parameters=[parameters.tolist()[0] for parameters in self.load_points],
+                log_likelihoods=log_likelihoods,
+                log_priors=log_priors,
+                weights=weights,
+                model=model
+            ),
+            time=self.timer.time
         )
 
     @property
     def load_total_iterations(self):
         with open(
-            "{}/{}.pickle".format(self.paths.samples_path, "total_iterations"), "rb"
+                "{}/{}.pickle".format(self.paths.samples_path, "total_iterations"), "rb"
         ) as f:
             return pickle.load(f)
 
@@ -348,27 +354,30 @@ class AbstractPySwarms(AbstractOptimizer):
     @property
     def load_log_posteriors(self):
         with open(
-            "{}/{}.pickle".format(self.paths.samples_path, "log_posteriors"), "rb"
+                "{}/{}.pickle".format(self.paths.samples_path, "log_posteriors"), "rb"
         ) as f:
             return pickle.load(f)
 
 
 class PySwarmsGlobal(AbstractPySwarms):
+
+    @convert_paths
     def __init__(
-        self,
-        paths=None,
-        sigma=3,
-        n_particles=None,
-        iters=None,
-        cognitive=None,
-        social=None,
-        inertia=None,
-        ftol=None,
-        initializer=None,
-        iterations_per_update=None,
-        number_of_cores=None,
+            self,
+            paths=None,
+            prior_passer=None,
+            n_particles=None,
+            iters=None,
+            cognitive=None,
+            social=None,
+            inertia=None,
+            ftol=None,
+            initializer=None,
+            iterations_per_update=None,
+            remove_state_files_at_end=None,
+            number_of_cores=None,
     ):
-        """ Class to setup and run a PySwarms Particle Swarm Optimizer global non-linear search.
+        """ A PySwarms Particle Swarm Optimizer global non-linear search.
 
         For a full description of PySwarms, checkout its Github and readthedocs webpages:
 
@@ -409,11 +418,9 @@ class PySwarmsGlobal(AbstractPySwarms):
         Parameters
         ----------
         paths : af.Paths
-            A class that manages all paths, e.g. where the phase outputs are stored, the non-linear search samples,
-            backups, etc.
-        sigma : float
-            The error-bound value that linked Gaussian prior withs are computed using. For example, if sigma=3.0,
-            parameters will use Gaussian Priors with widths coresponding to errors estimated at 3 sigma confidence.
+            Manages all paths, e.g. where the search outputs are stored, the samples, etc.
+        prior_passer : af.PriorPasser
+            Controls how priors are passed from the results of this `NonLinearSearch` to a subsequent non-linear search.
         n_particles : int
             The number of particles in the swarm used to sample parameter space.
         iters : int
@@ -439,7 +446,7 @@ class PySwarmsGlobal(AbstractPySwarms):
 
         super().__init__(
             paths=paths,
-            sigma=sigma,
+            prior_passer=prior_passer,
             n_particles=n_particles,
             iters=iters,
             cognitive=cognitive,
@@ -457,6 +464,8 @@ class PySwarmsGlobal(AbstractPySwarms):
         """Get the static Dynesty sampler which performs the non-linear search, passing it all associated input Dynesty
         variables."""
 
+        import pyswarms
+
         return pyswarms.global_best.GlobalBestPSO(
             n_particles=self.n_particles,
             dimensions=model.prior_count,
@@ -468,23 +477,26 @@ class PySwarmsGlobal(AbstractPySwarms):
 
 
 class PySwarmsLocal(AbstractPySwarms):
+
+    @convert_paths
     def __init__(
-        self,
-        paths=None,
-        sigma=3,
-        n_particles=None,
-        iters=None,
-        cognitive=None,
-        social=None,
-        inertia=None,
-        number_of_k_neighbors=None,
-        minkowski_p_norm=None,
-        ftol=None,
-        initializer=None,
-        iterations_per_update=None,
-        number_of_cores=None,
+            self,
+            paths=None,
+            prior_passer=None,
+            n_particles=None,
+            iters=None,
+            cognitive=None,
+            social=None,
+            inertia=None,
+            number_of_k_neighbors=None,
+            minkowski_p_norm=None,
+            ftol=None,
+            initializer=None,
+            iterations_per_update=None,
+            remove_state_files_at_end=None,
+            number_of_cores=None,
     ):
-        """ Class to setup and run a PySwarms Particle Swarm Optimizer global non-linear search.
+        """ A PySwarms Particle Swarm Optimizer global non-linear search.
 
         For a full description of PySwarms, checkout its Github and readthedocs webpages:
 
@@ -529,11 +541,9 @@ class PySwarmsLocal(AbstractPySwarms):
         Parameters
         ----------
         paths : af.Paths
-            A class that manages all paths, e.g. where the phase outputs are stored, the non-linear search samples,
-            backups, etc.
-        sigma : float
-            The error-bound value that linked Gaussian prior withs are computed using. For example, if sigma=3.0,
-            parameters will use Gaussian Priors with widths coresponding to errors estimated at 3 sigma confidence.
+            Manages all paths, e.g. where the search outputs are stored, the samples, etc.
+        prior_passer : af.PriorPasser
+            Controls how priors are passed from the results of this `NonLinearSearch` to a subsequent non-linear search.
         n_particles : int
             The number of particles in the swarm used to sample parameter space.
         iters : int
@@ -563,20 +573,20 @@ class PySwarmsLocal(AbstractPySwarms):
         """
 
         self.number_of_k_neighbors = (
-            self.config("search", "number_of_k_neighbors", int)
+            self._config("search", "number_of_k_neighbors")
             if number_of_k_neighbors is None
             else number_of_k_neighbors
         )
 
         self.minkowski_p_norm = (
-            self.config("search", "minkowski_p_norm", int)
+            self._config("search", "minkowski_p_norm")
             if minkowski_p_norm is None
             else minkowski_p_norm
         )
 
         super().__init__(
             paths=paths,
-            sigma=sigma,
+            prior_passer=prior_passer,
             n_particles=n_particles,
             iters=iters,
             cognitive=cognitive,
@@ -594,6 +604,8 @@ class PySwarmsLocal(AbstractPySwarms):
         """Get the static Dynesty sampler which performs the non-linear search, passing it all associated input Dynesty
         variables."""
 
+        import pyswarms
+
         return pyswarms.local_best.LocalBestPSO(
             n_particles=self.n_particles,
             dimensions=model.prior_count,
