@@ -1,26 +1,20 @@
 import logging
+from os import path
 import pickle
 from abc import ABC, abstractmethod
-from copy import deepcopy
 from typing import Dict
 
-import dill
-
 from autoconf import conf
 from autofit.mapper.model_mapper import ModelMapper
-from autofit.non_linear.paths import convert_paths
 from autofit.mapper.prior.promise import PromiseResult
-from autofit.non_linear import grid_search
-from autofit.non_linear.paths import Paths
+from autofit.non_linear.grid import grid_search
 
 logger = logging.getLogger(__name__)
 
 
 class AbstractPhase:
-    @convert_paths
     def __init__(
             self,
-            paths: Paths,
             *,
             search,
             model=None,
@@ -34,22 +28,31 @@ class AbstractPhase:
         search: class
             The class of a non_linear search
         """
-
-        self.search = deepcopy(search)
+        self.search = search
         self.model = model or ModelMapper()
 
         self.pipeline_name = None
         self.pipeline_tag = None
-        self.meta_dataset = None
 
     @property
     def paths(self):
         return self.search.paths
 
-    def save_model_info(self):
-        """Save the model.info file, which summarizes every parameter and prior."""
-        with open(self.paths.file_model_info, "w+") as f:
-            f.write(self.model.info)
+    @property
+    def folders(self):
+        return self.search.path_prefix
+
+    @property
+    def phase_property_collections(self):
+        """
+        Returns
+        -------
+        phase_property_collections: [PhaseProperty]
+            A list of phase property collections associated with this phase. This is
+            used in automated prior passing and should be overridden for any phase that
+            contains its own PhasePropertys.
+        """
+        return []
 
     @property
     def _default_metadata(self) -> Dict[str, str]:
@@ -64,80 +67,18 @@ class AbstractPhase:
             "pipeline_tag": self.pipeline_tag,
         }
 
-    def make_metadata_text(self, dataset_name):
-        return "\n".join(
-            f"{key}={value or ''}"
-            for key, value
-            in {
-                **self._default_metadata,
-                "dataset_name": dataset_name
-            }.items()
-        )
-
-    def save_metadata(self, dataset):
-        """
-        Save metadata associated with the phase, such as the name of the pipeline, the
-        name of the phase and the name of the dataset being fit
-        """
-        with open("{}/metadata".format(self.paths.make_path()), "w+") as f:
-            f.write(
-                self.make_metadata_text(
-                    dataset.name
-                )
-            )
-
-    def save_dataset(self, dataset):
-        """
-        Save the dataset associated with the phase
-        """
-        with open(f"{self.paths.pickle_path}/dataset.pickle", "wb") as f:
-            pickle.dump(dataset, f)
-
-    def save_mask(self, mask):
-        """
-        Save the mask associated with the phase
-        """
-        with open(f"{self.paths.pickle_path}/mask.pickle", "wb") as f:
-            dill.dump(mask, f)
-
-    def save_meta_dataset(self, meta_dataset):
-        with open(
-                f"{self.paths.pickle_path}/meta_dataset.pickle",
-                "wb+"
-        ) as f:
-            try:
-                pickle.dump(
-                    meta_dataset, f
-                )
-            except AttributeError:
-                pickle.dump(
-                    meta_dataset, f
-                )
-
-    def save_settings(self, settings):
-        with open(
-                f"{self.paths.pickle_path}/settings.pickle",
-                "wb+"
-        ) as f:
-            pickle.dump(
-                settings, f
-            )
-
-    def save_phase_attributes(self, phase_attributes):
-        with open(
-                f"{self.paths.pickle_path}/phase_attributes.pickle",
-                "wb+"
-        ) as f:
-            pickle.dump(
-                phase_attributes, f
-            )
-
     def __str__(self):
         return self.search.paths.name
 
     def __repr__(self):
         return f"<{self.__class__.__name__} {self.search.paths.name}>"
 
+    def run(self, dataset, mask, results=None):
+        raise NotImplementedError()
+
+    def modify_search_paths(self):
+        raise NotImplementedError()
+
     @property
     def result(self) -> PromiseResult:
         """
@@ -147,17 +88,18 @@ class AbstractPhase:
         """
         return PromiseResult(self)
 
-    def run_analysis(self, analysis, info=None):
-        return self.search.fit(model=self.model, analysis=analysis, info=info)
+    def run_analysis(self, analysis, info=None, pickle_files=None, log_likelihood_cap=None):
 
-    def make_phase_attributes(self, analysis):
+        return self.search.fit(model=self.model, analysis=analysis, info=info, pickle_files=pickle_files, log_likelihood_cap=log_likelihood_cap)
+
+    def make_attributes(self, analysis):
         raise NotImplementedError()
 
     def make_result(self, result, analysis):
         raise NotImplementedError()
 
     @property
-    def phase_name(self):
+    def name(self):
         return self.paths.name
 
 
@@ -192,16 +134,14 @@ class Dataset(ABC):
 
 
 class Phase(AbstractPhase):
-    @convert_paths
     def __init__(
             self,
-            paths,
             *,
             analysis_class,
             search,
             model=None,
     ):
-        super().__init__(paths, search=search, model=model)
+        super().__init__(search=search, model=model)
         self.analysis_class = analysis_class
 
     def make_result(self, result, analysis):
@@ -210,7 +150,7 @@ class Phase(AbstractPhase):
     def make_analysis(self, dataset):
         return self.analysis_class(dataset)
 
-    def run(self, dataset: Dataset, results=None, info=None):
+    def run(self, dataset: Dataset, results=None, info=None, pickle_files=None, log_likelihood_cap=None):
         """
         Run this phase.
 
@@ -226,21 +166,19 @@ class Phase(AbstractPhase):
         result: AbstractPhase.Result
             A result object comprising the best fit model and other hyper_galaxies.
         """
-        self.save_metadata(dataset=dataset)
-        self.save_dataset(dataset=dataset)
 
         self.model = self.model.populate(results)
 
         analysis = self.make_analysis(dataset=dataset)
 
-        result = self.run_analysis(analysis=analysis, info=info)
+        result = self.run_analysis(analysis=analysis, info=info, pickle_files=pickle_files, log_likelihood_cap=log_likelihood_cap)
 
         return self.make_result(result=result, analysis=None)
 
 
 def as_grid_search(phase_class, parallel=False):
     """
-    Create a grid search phase class from a regular phase class. Instead of the phase
+        Returns a grid search phase class from a regular phase class. Instead of the phase
     being optimised by a single non-linear optimiser, a new optimiser is created for
     each square in a grid.
 
@@ -261,17 +199,14 @@ def as_grid_search(phase_class, parallel=False):
     """
 
     class GridSearchExtension(phase_class):
-        @convert_paths
         def __init__(
                 self,
-                paths,
                 *,
                 search,
                 number_of_steps=4,
                 **kwargs,
         ):
-
-            super().__init__(paths, search=search, **kwargs)
+            super().__init__(search=search, **kwargs)
 
             self.search = grid_search.GridSearch(
                 paths=self.paths,
@@ -282,7 +217,7 @@ def as_grid_search(phase_class, parallel=False):
 
         def save_grid_search_result(self, grid_search_result):
             with open(
-                    f"{self.paths.pickle_path}/grid_search_result.pickle",
+                    path.join(self.paths.pickle_path, "grid_search_result.pickle"),
                     "wb+"
             ) as f:
                 pickle.dump(
@@ -292,6 +227,7 @@ def as_grid_search(phase_class, parallel=False):
         # noinspection PyMethodMayBeStatic,PyUnusedLocal
         def make_result(self, result, analysis):
             self.save_grid_search_result(grid_search_result=result)
+            open(self.paths.has_completed_path, "w+").close()
 
             return self.Result(
                 samples=result.samples,
@@ -301,6 +237,9 @@ def as_grid_search(phase_class, parallel=False):
             )
 
         def run_analysis(self, analysis, **kwargs):
+            self.search.search.paths = self.paths
+            self.search.paths = self.paths
+
             return self.search.fit(model=self.model, analysis=analysis, grid_priors=self.grid_priors)
 
         @property
@@ -313,7 +252,7 @@ def as_grid_search(phase_class, parallel=False):
     return GridSearchExtension
 
 
-class AbstractPhaseSettings:
+class AbstractSettingsPhase:
 
     def __init__(self, log_likelihood_cap=None):
 
@@ -332,8 +271,5 @@ class AbstractPhaseSettings:
         """
         if self.log_likelihood_cap is None:
             return ""
-        return (
-            "__"
-            + conf.instance.tag.get("phase", "log_likelihood_cap", str)
-            + "_{0:.1f}".format(self.log_likelihood_cap)
-        )
\ No newline at end of file
+        return f"__{conf.instance['notation']['settings_tags']['phase']['log_likelihood_cap']}" \
+               + "_{0:.1f}".format(self.log_likelihood_cap)
