@@ -6,10 +6,8 @@ import pytest
 import autofit as af
 from autofit import exc
 from autofit.text import formatter as frm
-from test_autofit import mock
-from test_autofit import mock_real
-
-dataset_path = "{}/../".format(os.path.dirname(os.path.realpath(__file__)))
+from autofit.mock import mock
+from autofit.mock import mock_real
 
 
 @pytest.fixture(name="initial_model")
@@ -70,8 +68,8 @@ class TestGenerateModelInfo:
         model_info = mm.info
 
         assert (
-                model_info
-                == """mock_class
+            model_info
+            == """mock_class
     one                                                                                   UniformPrior, lower_limit = 0.0, upper_limit = 1.0
     two                                                                                   UniformPrior, lower_limit = 0.0, upper_limit = 2.0"""
         )
@@ -85,42 +83,34 @@ class TestGenerateModelInfo:
         model_info = mm.info
 
         assert (
-                model_info
-                == """mock_class
+            model_info
+            == """mock_class
     one                                                                                   UniformPrior, lower_limit = 0.0, upper_limit = 1.0
     two                                                                                   1.0"""
         )
 
     def test_with_promise(self):
         mm = af.ModelMapper()
-        mm.promise = af.prior.Promise(
-            af.Phase(
-                phase_name="phase",
-                analysis_class=None,
-                search=af.MockSearch()
-            ),
+        mm.promise = af.Promise(
+            af.Phase(analysis_class=None, search=af.MockSearch("phase")),
             "path",
             result_path=[],
-            assert_exists=False
+            assert_exists=False,
         )
 
-        assert mm.info == 'promise                                                                                   phase.result.model.path'
+        assert (
+            mm.info
+            == "promise                                                                                   phase.result.model.path"
+        )
 
     def test_with_tuple(self):
         mm = af.ModelMapper()
         mm.tuple = (0, 1)
 
-        assert mm.info == 'tuple                                                                                     (0, 1)'
-
-
-class WithFloat:
-    def __init__(self, value):
-        self.value = value
-
-
-class WithTuple:
-    def __init__(self, tup=(0.0, 0.0)):
-        self.tup = tup
+        assert (
+            mm.info
+            == "tuple                                                                                     (0, 1)"
+        )
 
 
 # noinspection PyUnresolvedReferences
@@ -155,8 +145,8 @@ class TestRegression:
         assert isinstance(mm.sersic.centre_1, af.Prior)
 
     def test_tuple_parameter(self, mapper):
-        mapper.with_float = WithFloat
-        mapper.with_tuple = WithTuple
+        mapper.with_float = mock.WithFloat
+        mapper.with_tuple = mock.WithTuple
 
         assert mapper.prior_count == 3
 
@@ -171,7 +161,7 @@ class TestRegression:
 
         mm.one.one.id = mm.two.three.id + 1
 
-        assert mm.parameter_names == [
+        assert mm.model_component_and_parameter_names == [
             "one_two",
             "one_three",
             "two_one",
@@ -180,6 +170,13 @@ class TestRegression:
             "one_one",
         ]
 
+    def test_parameter_name_list(self):
+        mm = af.ModelMapper()
+        mm.one = mock.MockClassRelativeWidth
+        mm.two = mock.MockClassRelativeWidth
+
+        assert mm.parameter_names == ["one", "two", "three", "one", "two", "three"]
+
     def test_parameter_name_distinction(self):
         mm = af.ModelMapper()
         mm.ls = af.CollectionPriorModel(
@@ -188,7 +185,7 @@ class TestRegression:
                 af.PriorModel(mock.MockClassRelativeWidth),
             ]
         )
-        assert mm.parameter_names == [
+        assert mm.model_component_and_parameter_names == [
             "ls_0_one",
             "ls_0_two",
             "ls_0_three",
@@ -197,6 +194,27 @@ class TestRegression:
             "ls_1_three",
         ]
 
+    def test__parameter_labels(self):
+        mm = af.ModelMapper()
+        mm.one = mock.MockClassRelativeWidth
+        mm.two = mock.MockClassRelativeWidth
+
+        assert mm.parameter_labels == [
+            "one_label",
+            "two_label",
+            "three_label",
+            "one_label",
+            "two_label",
+            "three_label",
+        ]
+
+    def test__subscripts_of_parameters(self):
+        mm = af.ModelMapper()
+        mm.one = mock.MockClassRelativeWidth
+        mm.two = mock.MockClassRelativeWidth
+
+        assert mm.subscripts == ['o', 't', 't', 'o', 't', 't']
+
     def test_name_for_prior(self):
         ls = af.CollectionPriorModel(
             [
@@ -207,8 +225,8 @@ class TestRegression:
         assert ls.name_for_prior(ls[1].one) == "1_one"
 
     def test_tuple_parameter_float(self, mapper):
-        mapper.with_float = WithFloat
-        mapper.with_tuple = WithTuple
+        mapper.with_float = mock.WithFloat
+        mapper.with_tuple = mock.WithTuple
 
         mapper.with_float.value = 1.0
 
@@ -260,15 +278,11 @@ class TestRealClasses:
             [1 for _ in range(len(mapper.prior_tuples_ordered_by_id))]
         )
 
-        assert isinstance(
-            model_map.source_light_profile, mock_real.EllipticalSersic
-        )
+        assert isinstance(model_map.source_light_profile, mock_real.EllipticalSersic)
         assert isinstance(
             model_map.lens_mass_profile, mock_real.EllipticalCoredIsothermal
         )
-        assert isinstance(
-            model_map.lens_light_profile, mock_real.EllipticalCoreSersic
-        )
+        assert isinstance(model_map.lens_light_profile, mock_real.EllipticalCoreSersic)
 
     def test_attribute(self):
         mm = af.ModelMapper()
@@ -313,19 +327,11 @@ class TestConfigFunctions:
             [0.5 for _ in range(len(mapper.prior_tuples_ordered_by_id))]
         )
 
-        assert isinstance(
-            model_map.elliptical_profile_1, mock_real.EllipticalProfile
-        )
-        assert isinstance(
-            model_map.elliptical_profile_2, mock_real.EllipticalProfile
-        )
-        assert isinstance(
-            model_map.spherical_profile, mock_real.SphericalProfile
-        )
+        assert isinstance(model_map.elliptical_profile_1, mock_real.EllipticalProfile)
+        assert isinstance(model_map.elliptical_profile_2, mock_real.EllipticalProfile)
+        assert isinstance(model_map.spherical_profile, mock_real.SphericalProfile)
 
-        assert isinstance(
-            model_map.sersic_light_profile, mock_real.EllipticalSersic
-        )
+        assert isinstance(model_map.sersic_light_profile, mock_real.EllipticalSersic)
         assert isinstance(
             model_map.exponential_light_profile, mock_real.EllipticalExponential
         )
@@ -397,7 +403,7 @@ class TestModelInstancesRealClasses:
         assert model_map.profile_3.phi == 1.0
 
     def test__check_order_for_different_unit_values_and_set_priors_equal_to_one_another(
-            self
+        self
     ):
         mapper = af.ModelMapper(
             profile_1=mock_real.EllipticalProfile,
@@ -520,13 +526,13 @@ class TestModelInstancesRealClasses:
 
         np.random.seed(1)
 
-        assert mapper.random_unit_vector_within_limits(lower_limit=0.0, upper_limit=1.0) == pytest.approx(
-            [0.41702, 0.720324], 1.0e-4
-        )
+        assert mapper.random_unit_vector_within_limits(
+            lower_limit=0.0, upper_limit=1.0
+        ) == pytest.approx([0.41702, 0.720324], 1.0e-4)
 
-        assert mapper.random_unit_vector_within_limits(lower_limit=0.2, upper_limit=0.8) == pytest.approx(
-            [0.200068, 0.38140], 1.0e-4
-        )
+        assert mapper.random_unit_vector_within_limits(
+            lower_limit=0.2, upper_limit=0.8
+        ) == pytest.approx([0.200068, 0.38140], 1.0e-4)
 
     def test_random_vector_from_prior_within_limits(self):
         np.random.seed(1)
@@ -534,15 +540,21 @@ class TestModelInstancesRealClasses:
         mapper = af.ModelMapper()
         mapper.mock_class = af.PriorModel(mock.MockClassx2)
 
-        vector = mapper.random_vector_from_priors_within_limits(lower_limit=0.499999, upper_limit=0.500001)
+        vector = mapper.random_vector_from_priors_within_limits(
+            lower_limit=0.499999, upper_limit=0.500001
+        )
 
         assert vector == pytest.approx([0.5, 1.0], 1.0e-4)
 
-        vector = mapper.random_vector_from_priors_within_limits(lower_limit=0.899999, upper_limit=0.900001)
+        vector = mapper.random_vector_from_priors_within_limits(
+            lower_limit=0.899999, upper_limit=0.900001
+        )
 
         assert vector == pytest.approx([0.9, 1.8], 1.0e-4)
 
-        vector = mapper.random_vector_from_priors_within_limits(lower_limit=0.2, upper_limit=0.3)
+        vector = mapper.random_vector_from_priors_within_limits(
+            lower_limit=0.2, upper_limit=0.3
+        )
 
         assert vector == pytest.approx([0.21467, 0.4184677], 1.0e-4)
 
@@ -581,12 +593,12 @@ class TestUtility:
 
         assert len(mapper.prior_prior_model_dict) == 2
         assert (
-                mapper.prior_prior_model_dict[mapper.prior_tuples_ordered_by_id[0][1]].cls
-                == mock.MockClassx2
+            mapper.prior_prior_model_dict[mapper.prior_tuples_ordered_by_id[0][1]].cls
+            == mock.MockClassx2
         )
         assert (
-                mapper.prior_prior_model_dict[mapper.prior_tuples_ordered_by_id[1][1]].cls
-                == mock.MockClassx2
+            mapper.prior_prior_model_dict[mapper.prior_tuples_ordered_by_id[1][1]].cls
+            == mock.MockClassx2
         )
 
     def test_name_for_prior(self):
@@ -704,7 +716,7 @@ class TestListPriorModel:
         assert gaussian_mapper.list[1].two.sigma == 5
 
     def test_prior_results_for_gaussian_tuples__include_override_from_width_file(
-            self, list_prior_model
+        self, list_prior_model
     ):
         mapper = af.ModelMapper()
         mapper.list = list_prior_model
@@ -759,7 +771,7 @@ class Testinstance:
     def test_instance_in_config(self):
         mapper = af.ModelMapper()
 
-        mock_with_instance = af.PriorModel(MockClassx2instance)
+        mock_with_instance = af.PriorModel(MockClassx2instance, one=3)
 
         mapper.mock_class = mock_with_instance
 
@@ -862,14 +874,10 @@ class TestGaussianWidthConfig:
 @pytest.fixture(name="promise_mapper")
 def make_promise_mapper():
     mapper = af.ModelMapper()
-    mapper.model_component = af.PriorModel(
+    mapper.component = af.PriorModel(
         mock.MockComponents,
-        parameter=af.prior.Promise(
-            None,
-            None,
-            is_instance=False,
-            result_path=None,
-            assert_exists=False
+        parameter=af.Promise(
+            None, None, is_instance=False, result_path=None, assert_exists=False
         ),
     )
     return mapper
@@ -880,7 +888,7 @@ class TestPromises:
         assert promise_mapper.promise_count == 1
         assert promise_mapper.variable_promise_count == 1
 
-        promise_mapper.model_component.parameter.is_instance = True
+        promise_mapper.component.parameter.is_instance = True
         assert promise_mapper.variable_promise_count == 0
 
     def test_raises(self, promise_mapper):
