@@ -1,5 +1,4 @@
-
-from test_autofit.mock import MockDataset
+from autofit.mock.mock import MockDataset
 
 
 # def test_load():
